// RptPage.cpp : implementation file
//
// CrackUp version 1.0
// Copyright  1998 Ziff-Davis, Inc.
// First published in PC Magazine, US Edition, November 3, 1998
// Written by Bob Flanders and Gregory A. Wolking

#include "stdafx.h"
#include "crackup.h"
#include "RptPage.h"
#include "FileDlg.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////
// Global variable declarations (defined in CrackUp.cpp)
extern CDialog*		g_pRptWin;			// pointer to Property Sheet
extern BOOL			g_bNewWindows,		// flag for Windows version
					g_bMessageActive,	// flag for tray icon handler
					g_bStopScanNow,
					g_bAtLeastOne,		// false until one drive has been scanned.
					g_bScanInProgress;
extern WORD			g_wNDrivesSelected,	// number of drives selected
					g_wNDrives,			// total number of drives found
					g_wMaxFrag;			// value for most fragmented drive
extern DRVTABENT	g_stDrvTbl[];		// drive table 
extern HWND			g_hMasterWindow;	// Handle to main app window.
extern CRITICAL_SECTION g_csScanning;

////////////////
// Global function declarations (defined in CrackUp.cpp)
extern float		DefragsPct(PDRVTABENT pD, int iFormula);

////////////////////
// Format strings for DoReport()
const char* szFmt1 = "%15lu %s\n";
const char* szFmt2 = "%15lu %s\n\n";

// String indexes for detail report.
enum StringIndex
{
	eFiles,
	eDirs,
	eSystemFiles,
	eZero,
	eSysFrag,
	eSysOK,
	eFrag,
	eOK,
	eExtraFragments,
	eExtraSysFragments,
	eBoundedFreeChunks,
	eFreeChunks,
	eClusters,
	eSystemClusters,
	eClusInFrags,
	eClusUnFrags,
	eBFreeClusters,
	eUFreeClusters,
	ePercent0,
	ePercent1,
	eEndOfList,
};


// Array of strings for Detail Report.
const char * const szLabels[eEndOfList] =
{	
	"Moveable files",
	"Directories",
	"Unmoveable files",
	"Zero-length files",
	"Unmovable fragmented files",
	"Unmovable unfragmented files",
	"Moveable fragmented files and directories",
	"Moveable unfragmented files and directories",
	"Moveable extra fragments (MEF)",
	"Unmovable extra fragments (UEF)",
	"Bounded free fragments (BFF)",
	"Unbounded free fragments (UFF)",
	"Total clusters (TC)",
	"Unmovable clusters (UC)",
	"Moveable fragmented allocated clusters",
	"Moveable unfragmented allocated clusters",
	"Bounded free clusters",
	"Unbounded free clusters (UFC)",
	"Fragmentation (moveable files only)",
	"Fragmentation           (all files)",
};


/////////////////////////////////////////////////////////////////////////////
// CRptPage property page

IMPLEMENT_DYNCREATE(CRptPage, CPropertyPage)

CRptPage::CRptPage() : CPropertyPage(CRptPage::IDD)
{
	//{{AFX_DATA_INIT(CRptPage)
	m_intFormula = 0;
	//}}AFX_DATA_INIT
}

CRptPage::~CRptPage()
{
}

void CRptPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRptPage)
	DDX_Control(pDX, IDC_cmdScanNow, m_cmdScanNow);
	DDX_Control(pDX, IDC_cmdGenerate, m_cmdGenerate);
	DDX_Control(pDX, IDC_lblUnfrag1, m_lblUnfrag1);
	DDX_Control(pDX, IDC_lblIncomplete, m_lblIncomplete);
	DDX_Control(pDX, IDC_lblFrag1, m_lblFrag1);
	DDX_Control(pDX, IDC_boxPieChart, m_boxPieChart);
	DDX_Control(pDX, IDC_boxUnfrag, m_boxUnfrag);
	DDX_Control(pDX, IDC_boxFrag, m_boxFrag);
	DDX_Control(pDX, IDC_lblTitle, m_lblTitle);
	DDX_Control(pDX, IDC_lblUnfrag, m_lblUnfrag);
	DDX_Control(pDX, IDC_lblFrag, m_lblFrag);
	DDX_Control(pDX, IDC_lstRptDrives, m_lstDrives);
	DDX_Radio(pDX, IDC_optFormula1, m_intFormula);
	//}}AFX_DATA_MAP
	DDX_Control(pDX, IDC_boxPieChart, m_boxPieChart);
}


BEGIN_MESSAGE_MAP(CRptPage, CPropertyPage)
	//{{AFX_MSG_MAP(CRptPage)
	ON_BN_CLICKED(IDC_cmdGenerate, OncmdGenerate)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_lstRptDrives, OnItemchangedList1)
	ON_NOTIFY(LVN_GETDISPINFO, IDC_lstRptDrives, OnGetdispinfoList1)
	ON_BN_CLICKED(IDC_optFormula1, OnoptFormula1)
	ON_BN_CLICKED(IDC_cmdScanNow, OncmdScanNow)
	ON_BN_CLICKED(IDC_optFormula2, OnoptFormula1)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_COPY, OnCopy)
END_MESSAGE_MAP()

//////////////////////////////////
// CRptPage helper functions.


// **************************************************************************
// Find Selected List Entry
// **************************************************************************
int CRptPage::GetCurrentItem()
{
	int			iI;		// work variable
	UINT		uState;	// CListCtrl item state

	for (iI = 0; iI < m_lstDrives.GetItemCount(); iI++)			// for each item 
	{															//  in the CListCtrl
		uState = m_lstDrives.GetItemState(iI, LVIS_FOCUSED);	//  see if it has focus
		if (uState)												// q. does item have focus?
			break;												// a. yes .. leave the loop
	}												
	if (!uState)												// q. any item have focus?
		iI = 0;													// a. yes .. no, use first item
	return iI;													// return selected item
}


// **************************************************************************
// Find Selected Drive Entry
// **************************************************************************
DRVTABENT *CRptPage::GetCurrentSel()
{
	DRVTABENT	*pD = NULL;		// pointer to an entry
	int			iI, iJ, iCount;	// work variables
	UINT		uState;			// CListCtrl item state

	iCount = m_lstDrives.GetItemCount();			// Get # of items in list.
	if (iCount == 0)								// Is list empty?
		return NULL;								// Indicate no selection if so.
	for (iI = 0; iI < iCount; iI++)							// for each item 
	{														//  in the CListCtrl
		uState = m_lstDrives.GetItemState(iI, LVIS_FOCUSED);	//  see if it has focus
		if (uState)											// q. does item have focus?
			break;											// a. yes .. leave the loop
	}												
	if (!uState)											// q. any item have focus?
		iI = 0;												// a. yes .. no, use first item
	for (iJ = 0; iJ < g_wNDrives; iJ++)						// for each 
	{														//  eligable drive found
		if (g_stDrvTbl[iJ].bCheckDrive)						// q. is this drive checked?
		{
			if (!iI)										// a. yes .. is it the item?
			{												// a. yes ..
				pD = &g_stDrvTbl[iJ];							//  set the pointer to it's info
				break;										//  and go forth and draw it
			}			
			else											// Otherwise ..		
			{
				iI--;										//  check next item
			}
		}
	}
	return pD;												// return the table entry
}

// Create CRACKUP.TXT report and launch it if possible.
void CRptPage::DoReport()
{
	FILE		*fRpt;	// file for report
	WORD		wN;		// work number
	PDRVTABENT	pD;		// drive table entry
	SYSTEMTIME	stCurrent;	// System time structure.
	const char  szTitle[] = "CrackUp: Save Report As...";
	CString		strReportFile,				// Output filespec.
				strTime,					// Used to format time strings.
				strTemp;					// Scratch string.
	int			iResult,					// Used to hold 
				iCounter;
	BOOL		bCSV = FALSE,				// Flag for CSV format.
				bReportsReady = FALSE;		// Flag if any reports are ready.

	pD = &g_stDrvTbl[0];						// init drive pointer
	for (wN = g_wNDrives; wN--; pD++)			// for each drive ..
	{
		if (pD->bCheckDrive == 0)				// q Is this drive selected?
			continue;							// a. no .. next
		bReportsReady = pD->bRptRdy; 
		if (bReportsReady)						// q. This drive's report ready?
			break;								// a. no .. exit the loop.
	}
	if (!bReportsReady)					// Error message if no reports are ready.
	{
		g_bMessageActive = TRUE;
		MessageBox("No drives have been scanned yet.\n"
			"There is nothing to report.",
			"CrackUp Error", MB_ICONINFORMATION);
		g_bMessageActive = FALSE;
		return;
	}
	CMyFileDlg dlgFile(FALSE,								// Create a custom File Save As common dialog.
		"txt",												// default extension.
		"CrackUp Report",									// default filename.
		OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,				// No read only, prompt if file exists.
		"Text Files (*.txt)|*.txt|"							// Extension filters.
		"Comma-separated values (*.csv)|*.csv|"
		"All Files (*.*)|*.*||",
		this);												// We're the owner.
	dlgFile.m_ofn.lpstrTitle = szTitle;						// Set custom title.
	if (dlgFile.DoModal() == IDOK)							// Launch the dialog. If user said OK,
		strReportFile = dlgFile.GetPathName();				// retrieve the fully-qualified filename.
	else													// Otherwise,
		return;												// we're done.
	bCSV = dlgFile.m_ofn.nFilterIndex == 2;		// Set flag if user selected the "Comma-separated values" file type.
	fRpt = fopen(strReportFile, "w+t");			// open a text file, overwriting if it already exists.
	if (fRpt == NULL)							// File creation failed?
	{
		AfxFormatString1(strTemp, IDS_REPORT_FAIL, strReportFile);
		g_bMessageActive = TRUE;				// Set flag for tray icon handler.
		MessageBox(strTemp, "CrackUp Error", MB_ICONINFORMATION); // Display the message.
		g_bMessageActive = FALSE;				// Clear flag.
		return;
	}
	GetLocalTime(&stCurrent);
	CTime tCurrent(stCurrent);
	if (bCSV)		// CSV file type?
	{
		if (dlgFile.m_bHeaders)		// "Include column headers" box checked?
		{
			// Write header record if so.
			if ((iResult = 	fprintf(fRpt, "\"Drive\",\"Label\",\"File System\",\"Last Scanned\",")) > 0)
			{
				for (iCounter = eFiles; iCounter < eEndOfList - 1; iCounter++)
				{
					if ((iResult = fprintf(fRpt, "\"%s\",", szLabels[iCounter])) < 0)
						break;
				}
				if (iResult > 0)
				iResult = fprintf(fRpt, "\"%s\"\n", szLabels[iCounter]);
			}
		}
	}
	else
	{
		strTime = tCurrent.Format("Generated %#c");
		iResult = fprintf(fRpt,									// print the header
				"CRACKUP Report: Windows 95%s\n",				//  inserting
				(g_bNewWindows ? "(b)" : ""));					//  95b header if needed
		if (iResult > 0)
			iResult = fprintf(fRpt, "%s\n\n", strTime);
	}
	if (iResult > 0)
	{
		pD = &g_stDrvTbl[0];						// init drive pointer
		for (wN = g_wNDrives; wN--; pD++)			// for each drive ..
		{
			if (!pD->bCheckDrive)					// q Is this drive selected?
				continue;							// a. no .. next drive.
			if (!pD->bRptRdy)						// q Is this drive's report ready?
				continue;							// a. no .. next drive.
			CTime tLast(pD->stLastScan, -1);		// Retrieve last scan.
			if (bCSV)
			{
				iResult = fprintf(fRpt,
						"\"%c:\",\"%s\",\"FAT%u\",\"%s\",",
						pD->cDrv, pD->acVolName, pD->wFatType, strTime);
				if (iResult < 0)
					break;
				iResult = fprintf(fRpt,
							"%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%.1f%%,%.1f%%\n",
							pD->dwFiles, pD->dwDirs, pD->dwSystemFiles, pD->dwZero,
							pD->dwSysFrag, pD->dwSysOK, pD->dwFrag, pD->dwOK,
							pD->dwExtraFragments, pD->dwExtraSysFragments, pD->dwBoundedFreeChunks, pD->dwFreeChunks - pD->dwBoundedFreeChunks,
							pD->dwClusters, pD->dwSystemClusters, pD->dwClusInFrags, pD->dwClusUnFrags,
							pD->dwBFreeClusters, pD->dwUFreeClusters, DefragsPct(pD, 0) * 100, DefragsPct(pD,1) * 100);
				if (iResult < 0)
					break;
			}
			else
			{
				strTime = tLast.Format("Last scan: %#c");	// Format it as a string.
				if ((iResult = fprintf(fRpt, "Drive %c:%s (FAT%02.2d)\n", // Write section header for this drive.
						pD->cDrv, 
						pD->acVolName, 
						pD->wFatType)) < 0)
					break;
				if ((iResult = fprintf(fRpt, "\t%s\n", strTime)) < 0)
					break;
				// Write detail info for this drive.
				if ((iResult = fprintf(fRpt, szFmt1, pD->dwFiles, szLabels[eFiles])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt1, pD->dwDirs, szLabels[eDirs])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt1, pD->dwSystemFiles, szLabels[eSystemFiles])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt2, pD->dwZero, szLabels[eZero])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt1, pD->dwSysFrag, szLabels[eSysFrag])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt1, pD->dwSysOK, szLabels[eSysOK])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt1, pD->dwFrag, szLabels[eFrag])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt2, pD->dwOK, szLabels[eOK])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt1, pD->dwExtraFragments, szLabels[eExtraFragments])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt1, pD->dwExtraSysFragments, szLabels[eExtraSysFragments])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt1, pD->dwBoundedFreeChunks, szLabels[eBoundedFreeChunks])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt2, (pD->dwFreeChunks - pD->dwBoundedFreeChunks), szLabels[eFreeChunks])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt1, pD->dwClusters, szLabels[eClusters])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt1, pD->dwSystemClusters, szLabels[eSystemClusters])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt1, pD->dwClusInFrags, szLabels[eClusInFrags])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt1, pD->dwClusUnFrags, szLabels[eClusUnFrags])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt1, pD->dwBFreeClusters, szLabels[eBFreeClusters])) < 0)
					break;
				if ((iResult = fprintf(fRpt, szFmt2, pD->dwUFreeClusters, szLabels[eUFreeClusters])) < 0)
					break;
				if ((iResult = fprintf(fRpt, "\t*** %s: %.1f%%\n", szLabels[ePercent0], DefragsPct(pD, 0) * 100)) < 0)
					break;
				if ((iResult = fprintf(fRpt, "\t*** %s: %.1f%%\n\n", szLabels[ePercent1], DefragsPct(pD, 1) * 100)) < 0)
					break;

			}
		}
	}
	// Close the file.
	fclose(fRpt);
	// Check for error during write
	if (iResult < 0)
	{
		g_bMessageActive = TRUE;				// Set flag for tray icon handler.
		// Format and display error message.
		strTemp.Format("I/O error writing to\n\"%s\"\n\nReport is incomplete.", strReportFile);
		MessageBox(strTemp,	"CrackUp Error", MB_ICONINFORMATION);
		g_bMessageActive = FALSE;				// Clear flag.
	}
	// If user checked the "Launch report" box, try to launch the document.
	if (dlgFile.m_bLaunch)
		ShellExecute(NULL, "open", strReportFile, NULL, NULL, SW_SHOWNORMAL); 
}

/////////////////////////////////////////////////////////////////////////////
// CRptPage message handlers

// "Generate Detail Report" button.
void CRptPage::OncmdGenerate() 
{
	DoReport();
}

// "Copy results to clipboard" generated by the list control's context menu.

void CRptPage::OnCopy()
{
	PDRVTABENT pD;
	HGLOBAL hMyData = NULL;
	CString strPara, strTemp;
	char * szMyData = NULL;
	int i;


	strPara.Empty();
	for (i= 0; i < g_wNDrives; i++)		// for each drive ..
	{
		pD = &g_stDrvTbl[i];							// resolve entry address
		if (!pD->bCheckDrive)
			continue;
		if (pD->bRptRdy)
		{
			CTime tLast(pD->stLastScan);				// Retrieve time of scan.
			CString strLast = tLast.Format("%c");		// Convert to string format.
			strTemp.Format("%c: %s %.1f%% %.1f%% \r\n",
				pD->cDrv,
				strLast,
				DefragsPct(pD, 0) * 100,
				DefragsPct(pD, 1) * 100);
		}
		else
		{
			strTemp.Format("%c: has not been scanned.\r\n", pD->cDrv);
		}
		strPara += strTemp;
	}
	if (strPara.IsEmpty())
		strPara = "No drives have been scanned.\r\n";
	if (OpenClipboard())
	{
		hMyData = GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE | GMEM_ZEROINIT, strPara.GetLength() + 1);
		if (hMyData)
		{
			szMyData = (char*) GlobalLock(hMyData);
			lstrcpy(szMyData, strPara);
			GlobalUnlock(hMyData);
			SetClipboardData(CF_TEXT, hMyData);
		}
		CloseClipboard();
	}
}


// Called when this page becomes the active page in the Property Sheet.
BOOL CRptPage::OnSetActive() 
{
	PDRVTABENT	pZ;			// Pointer to drive table entry
	char		szS[30];	// work string
	int			iDrv,		// drive table entry
				iJ,			// work int
				iColWdth,	// column width
				iImg;		// image for this drive
	CString		strWrkLbl;	// work area for label
	LV_COLUMN	lvC;		// column structure
	LV_ITEM		lvi;		// structure for item insert
	RECT		rectX;		// client rectantle of clistctrl

	m_lstDrives.GetClientRect(&rectX);	// get the client rectangle
	if (!m_bFirstSetActive)				// If this is not the first time we've been activated,
	{
		m_lstDrives.DeleteAllItems();	// clear the list box.
	}
	else								// Otherwise,
	{									// initialize the list box:
		iColWdth = (rectX.right - 2) / 2;			// set up the width of the drive/label
		if ( m_ilDrives.Create( IDB_ICONS, 16, 1, RGB(128,128,128) ) )
			m_lstDrives.SetImageList( &m_ilDrives, LVSIL_SMALL);
		lvC.mask = LVCF_FMT							// format is valid
					| LVCF_TEXT						//  header is valid
					| LVCF_WIDTH
					| LVCF_SUBITEM;					//  subitem is valid
		lvC.fmt = LVCFMT_LEFT;						// set the column format
 		lvC.pszText = "Drive";						//  name
		lvC.cx = iColWdth;							//  width
		lvC.iSubItem = 0;							//  subitem number
		m_lstDrives.InsertColumn(0, &lvC);			// Make me a column!
		lvC.iSubItem = 1;
		lvC.pszText = "Last Scan";
		m_lstDrives.InsertColumn(1, &lvC);
	}
	iJ = 0;											// init list counter
	for (iDrv = 0; iDrv < g_wNDrives; iDrv++)		// for each drive ..
	{
		pZ = &g_stDrvTbl[iDrv];							// resolve entry address
		if (!pZ->bCheckDrive)							// q. drive checked?
			continue;									// a. no .. next
		sprintf(szS, "%c: %s",							// build the drive label
				pZ->cDrv,								//  drive letter:
				pZ->acVolName);							//  then label
		iImg = (pZ->fFragLvl-.10 >= g_wMaxFrag) ? 1:0;	// get image number
		strWrkLbl = szS;
		strWrkLbl.TrimRight();
		lvi.mask=LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM;
		lvi.iItem=iJ;
		lvi.iSubItem=0;
		lvi.pszText=(char *) (LPCTSTR) strWrkLbl;
		lvi.iImage=iImg;
		lvi.lParam = (LPARAM) pZ;
		m_lstDrives.InsertItem(&lvi);
		iJ++;	
	}
	lvi.mask=LVIF_STATE;
	lvi.iItem=0;
	lvi.iSubItem=0;
	lvi.state = 3;
	lvi.stateMask = 3;
	m_lstDrives.SetItemState(0, &lvi);
	m_lstDrives.SetFocus();
	m_intLastSelection = -1;
	UpdateData(FALSE);						// Update controls from member variables.
	UpdateLabels();
	return CPropertyPage::OnSetActive();	// Let base class update internal flags.
}

// Called when either radio button is clicked.
void CRptPage::OnoptFormula1() 
{
	UpdateData(TRUE);		// Get data from controls.
	UpdateLabels();
}

// "Scan Now" button.
void CRptPage::OncmdScanNow() 
{
	if (g_bScanInProgress)						// Scan running?
	{
		g_bStopScanNow = TRUE;					// Tell it to stop
		EnterCriticalSection(&g_csScanning);	// and wait until it does.
		LeaveCriticalSection(&g_csScanning);
	}
	// Tell main window to perform the scan.
	::PostMessage(g_hMasterWindow, WM_COMMAND, ID_SCANNOW, 0);	
}

// Called when listbox selection changes.
void CRptPage::OnItemchangedList1(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

	m_lstDrives.Update(pNMListView->iItem);
	m_lstDrives.UpdateWindow();
	UpdateLabels();
	*pResult = 0;
}

// Called when listbox needs info to display an item.
void CRptPage::OnGetdispinfoList1(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NMLVDISPINFO* pDispInfo = (NMLVDISPINFO*)pNMHDR;
	PDRVTABENT			pZ;
	static char			szS[30];

	if (pDispInfo->item.mask != LVIF_TEXT)	// We don't supply anything except the text.
	{
		*pResult = -1;
		return;
	}
	pZ = (PDRVTABENT) pDispInfo->item.lParam;	// Get pointer to current item's drive table entry.
	*pResult = 0;	// Default result = success.
	switch(pDispInfo->item.iSubItem)			// Branch based on subitem requested.
	{
	case 0:	// SubItem 0 (first column)
		sprintf(szS, "%c: %s",						// build the drive label
				pZ->cDrv,							//  drive letter:
				pZ->acVolName);						//  then label
		strcpy(pDispInfo->item.pszText,szS);		// save drive/label combo
		break;
	case 1:	// SubItem 1 (second column)
		if (pZ->bRptRdy)							// If drive was scanned,
		{
			CTime tLast(pZ->stLastScan);				// Retrieve time of scan.
			CString strLast = tLast.Format("%c");		// Convert to string format.
			if (strLast.IsEmpty())						// If time value was invalid,
				strcpy(pDispInfo->item.pszText, "Unknown");	// say so.
			else											// Otherwise,
				strcpy(pDispInfo->item.pszText, strLast);	// install the formatted string.
		}
		else										// Otherwise,
			strcpy(pDispInfo->item.pszText, "Never");	// Indicate drive has not been scanned yet.
		break;
	default:	// This should never be called, as control only has two columns.
		*pResult = -1;		// but just in case, signal that we don't know what this subitem is.
	}
}


BOOL CRptPage::OnInitDialog() 
{
	// Initialize rest of dialog controls.
	CPropertyPage::OnInitDialog();
	// Set colors for pie graph control.
	m_boxPieChart.m_crPrimary = FRAGCOLOR;
	m_boxPieChart.m_crSecondary = UNFRAGCOLOR;
	m_boxPieChart.m_crLines = RGB(0, 0, 0);
	m_boxPieChart.m_crBackground = RGB(255, 255, 255);
	// Set default percentage for pie graph.
	m_boxPieChart.SetPercentage(0.0f);
	// Set colors for legend boxes.
	m_boxFrag.m_crFaceColor = FRAGCOLOR;
	m_boxUnfrag.m_crFaceColor = UNFRAGCOLOR;
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

// Updates display when list box or
// radio button selection changes.
void CRptPage::UpdateLabels()
{
	float		fUnfrag,			// unfragged percentage|clusters
				fFrag;				// fragged percentage or clusters
	char		szMessage[50];		// messages
	PDRVTABENT  pD;					// currently selected drive
	UINT		nCmd1, nCmd2;		// Commands for showing/hiding static controls.

	pD = GetCurrentSel();								// get currently selected drive
	if (pD == NULL)										// If list is empty,
	{
		strcpy(szMessage, "No drives selected.");		// Set labels accordingly.
		m_lblTitle.SetWindowText(szMessage);
		m_lblIncomplete.SetWindowText(szMessage);
		m_boxPieChart.SetPercentage(0.0f);	// No percentage to show.
		nCmd1 = SW_SHOW;				// Show the "not scanned" label.
		nCmd2 = SW_HIDE;				// Hide the other static controls.
	}
	else
	{
		sprintf(szMessage, "Drive %c: %s", pD->cDrv, pD->acVolName);	// build the drive's identity
		m_lblTitle.SetWindowText(szMessage);							//  and put it above the pie
		if (pD->bRptRdy)									// q. report ready
		{													
			fFrag = DefragsPct(pD, m_intFormula);			// a. yes .. get the percentage
			m_boxPieChart.SetPercentage(fFrag);				// Redraw the graph.
			fFrag = ((int) (fFrag * 1000)) / 10.0f;			// truncate to one decimal place.
			fUnfrag = (100.0F - fFrag);						// calculate remainder
			sprintf(szMessage,								// show unfragmented space
				"%3.1f%%",									//  put in message
				fUnfrag);									//  and calculated percent
			m_lblUnfrag.SetWindowText(szMessage);			//  show the message
			sprintf(szMessage,								// show fragmented space
				"%3.1f%%",									//  put in message
				fFrag);										//  and calculated percent
			m_lblFrag.SetWindowText(szMessage);				//  show the message
			nCmd1 = SW_HIDE;	// Hide the "not scanned" label.
			nCmd2 = SW_SHOW;	// Show the other static controls.
		}
		else
		{
			m_boxPieChart.SetPercentage(0.0f);	// No percentage to show.
			m_lblIncomplete.SetWindowText("Drive not scanned.");
			nCmd1 = SW_SHOW;				// Show the "not scanned" label.
			nCmd2 = SW_HIDE;				// Hide the other static controls.
		}
	}
	// Show/Hide controls appropriately.
	m_lblIncomplete.ShowWindow(nCmd1);
	m_boxFrag.ShowWindow(nCmd2);
	m_boxUnfrag.ShowWindow(nCmd2);
	m_lblFrag.ShowWindow(nCmd2);
	m_lblFrag1.ShowWindow(nCmd2);
	m_lblUnfrag.ShowWindow(nCmd2);
	m_lblUnfrag1.ShowWindow(nCmd2);
	m_cmdGenerate.EnableWindow(g_wNDrivesSelected > 0 && g_bAtLeastOne);
	m_cmdScanNow.EnableWindow(g_wNDrivesSelected > 0);
}
